# -*- coding: utf-8 -*-
from Products.urban.interfaces import ICODT_BaseBuildLicence
from Products.urban.interfaces import IIntentionToSubmitAmendedPlans
from datetime import timedelta
from imio.schedule.content.logic import StartDate
from imio.schedule.interfaces import ICalculationDelay
from zope.component import queryMultiAdapter

from Products.urban.interfaces import ICODT_BaseBuildLicence


class AcknowledgmentLimitDate(StartDate):
    """
    Acknowledgment limit date is the deposit date + (20 or 30).
    If there is modified blueprints,
    the limit date is the old licence notification limit date.
    """

    def start_date(self):
        # XXX: executed 5 times at licence creation during test; why ?
        licence = self.task_container
        limit_date = None
        if hasattr(licence, "getHasModifiedBlueprints") and not licence.getHasModifiedBlueprints():
            deposit = licence.getLastDeposit()
            date = deposit and deposit.getEventDate()
            delay = 20
            if licence.is_CODT2024() is True:
                delay = 30
            limit_date = date and date + delay or None
        elif licence.getLastEvent(IIntentionToSubmitAmendedPlans):
            deposit_event = licence.getLastDeposit()
            intention_event = licence.getLastEvent(IIntentionToSubmitAmendedPlans)

            deposit_event_date = deposit_event.getEventDate()
            intention_receipt_date = intention_event.getReceiptDate()

            if not deposit_event or not deposit_event_date:  # not supposed to happen
                return limit_date

            deposit_event_delay = 30 if licence.is_CODT2024() is True else 20

            # shouldn't happen; just in case, do as if there were no intention
            if not intention_receipt_date:
                limit_date = deposit_event_date + deposit_event_delay

            # modified plans have been submitted
            if intention_receipt_date < deposit_event_date:
                date = deposit_event_date
                limit_date = date + deposit_event_delay
            else:
                # no modified plans received (either still waiting, or delay is over)
                ultime_date = intention_event.getUltimeDate()
                if ultime_date:
                    limit_date = ultime_date
                else:
                    receipt_date = intention_event.getReceiptDate()
                    if receipt_date:
                        limit_date = receipt_date + 180

                if not limit_date:
                    return None
                limit_date = limit_date + deposit_event_delay
        elif hasattr(licence, "getLastAcknowledgment"):
            # XXX Need review for CODT 2024
            ack = licence.getLastAcknowledgment(state="closed")
            annonced_delay = queryMultiAdapter(
                (licence, self.task),
                ICalculationDelay,
                "urban.schedule.delay.annonced_delay",
            )
            annonced_delay = annonced_delay and annonced_delay.calculate_delay(with_modified_blueprints=False) or 0
            limit_date = ack and ack.getEventDate() + annonced_delay
        return limit_date


class AmendedPlansLimitDate(StartDate):
    """
    Returns amended plans event's limit date, or its receipt date + 180 days
    """

    def start_date(self):
        licence = self.task_container
        event = licence.getLastEvent(IIntentionToSubmitAmendedPlans)
        limit_date = None

        if event:
            ultime_date = event.getUltimeDate()
            if ultime_date:
                limit_date = ultime_date
            else:
                receipt_date = event.getReceiptDate()
                if receipt_date:
                    limit_date = receipt_date + 180

        return limit_date


class FDOpinionLimitDate(StartDate):
    def start_date(self):
        licence = self.task_container
        limit_date = None
        if hasattr(licence, "getLastWalloonRegionOpinionRequest"):
            fd_opnion = licence.getLastWalloonRegionOpinionRequest()
            date = fd_opnion and fd_opnion.getEventDate()
            delay = 35
            if licence.is_CODT2024() is True:
                delay = 30
            limit_date = date and date + delay or None

        return limit_date


class PloneMeetingCollegeDecidedDate(StartDate):
    def start_date(self):
        licence = self.task_container
        if ICODT_BaseBuildLicence.providedBy(licence):
            return licence.get_last_college_date()


class PloneMeetingCouncilDecidedDate(StartDate):
    def start_date(self):
        licence = self.task_container
        if ICODT_BaseBuildLicence.providedBy(licence):
            return licence.get_last_council_date()



# -*- coding: utf-8 -*-

from pkg_resources import get_distribution

import sys

project = "urban.schedule"
copyright = "2023, iMio"

version = release = get_distribution(project).version


# The suffix of source filenames.
source_suffix = {
    ".md": "markdown",
    ".rst": "restructuredtext",
}

extensions = [
    "sphinx.ext.doctest",
    "sphinx.ext.coverage",
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.autosummary",
    "myst_parser",
    "sphinx.ext.todo",
]
master_doc = "index"

locale_dirs = ["translated/"]
language = "en"

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual])
# This enables PDF generation.
latex_documents = [
    (
        "index",
        "urban.schedule.tex",
        "urban.schedule Documentation",
        "",
        "manual",
    )
]


class Mock:
    def __init__(self, *args, **kwargs):
        pass

    def __call__(self, *args, **kwargs):
        return Mock()

    @classmethod
    def __getattr__(cls, name):
        if name in ("__file__", "__path__"):
            return "/dev/null"
        elif name[0] == name[0].upper():
            mockType = type(name, (), {})
            mockType.__module__ = __name__
            return mockType
        else:
            return Mock()


MOCK_MODULES = ["lxml"]
for mod_name in MOCK_MODULES:
    sys.modules[mod_name] = Mock()


# -- Options for myST markdown conversion to html -----------------------------

myst_enable_extensions = [
    "colon_fence",
    "deflist",
]


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_book_theme"



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from urban.schedule import testing  # noqa: E501

import unittest


class TestSetup(unittest.TestCase):
    """Test that urban.schedule is properly installed."""

    layer = testing.URBAN_SCHEDULE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if urban.schedule is installed."""
        self.assertTrue(self.installer.is_product_installed("urban.schedule"))

    def test_browserlayer(self):
        """Test that IUrbanScheduleLayer is registered."""
        from urban.schedule.interfaces import IUrbanScheduleLayer
        from plone.browserlayer import utils

        self.assertIn(IUrbanScheduleLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = testing.URBAN_SCHEDULE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.installer = api.portal.get_tool('portal_quickinstaller')
        roles_before = api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer.uninstall_product("urban.schedule")
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        """Test if urban.schedule is cleanly uninstalled."""
        self.assertFalse(self.installer.is_product_installed("urban.schedule"))

    def test_browserlayer_removed(self):
        """Test that IUrbanScheduleLayer is removed."""
        from urban.schedule.interfaces import IUrbanScheduleLayer
        from plone.browserlayer import utils

        self.assertNotIn(IUrbanScheduleLayer, utils.registered_layers())



# -*- coding: utf-8 -*-
from Products.urban.interfaces import ICODT_BaseBuildLicence
from Products.urban.interfaces import IIntentionToSubmitAmendedPlans
from imio.schedule.content.condition import CreationCondition
from Products.urban.interfaces import ICODT_BaseBuildLicence


class IsCODT2024(CreationCondition):
    """Validate that the current licence is impacted by the new CODT reform"""

    def evaluate(self):
        licence = self.task_container
        return licence.is_CODT2024()


class IsNotCODT2024(CreationCondition):
    """Validate that the current licence is not impacted by the new CODT reform"""

    def evaluate(self):
        licence = self.task_container
        return licence.is_CODT2024() is not True


class HasAmendedPlans(CreationCondition):
    """"""

    def evaluate(self):
        licence = self.task_container
        event = licence.getLastEvent(IIntentionToSubmitAmendedPlans)
        return event is not None


class IsPloneMeetingCollegeDone(CreationCondition):
    def evaluate(self):
        licence = self.task_container
        if ICODT_BaseBuildLicence.providedBy(licence):
            return True if licence.get_last_college_date() else False



class IsPloneMeetingCouncilDone(CreationCondition):
    def evaluate(self):
        licence = self.task_container
        if ICODT_BaseBuildLicence.providedBy(licence):
            return True if licence.get_last_council_date() else False



# -*- coding: utf-8 -*-

from urban.schedule import utils

import logging

logger = logging.getLogger("urban.events: migrations")


def import_schedule_config(context):
    utils.import_all_config()


def update_reception(context):
    logger.info("starting : Update reception tasks")
    if "standard" in utils.get_configs():
        utils.import_all_config(
            base_json_path="./profiles/config/standard",
            handle_existing_content=utils.ExistingContent.UPDATE,
            match_filename="reception.json",
        )
    else:
        logger.info("nothing to upgrade")
    logger.info("upgrade done!")


def update_fd_opinion(context):
    logger.info("starting : Update fd opinion tasks")
    if "liege" in utils.get_configs():
        utils.import_all_config(
            base_json_path="./profiles/config/liege",
            handle_existing_content=utils.ExistingContent.UPDATE,
            match_filename="avis-fd.json",
        )
    else:
        logger.info("nothing to upgrade")
    logger.info("upgrade done!")


def update_reception_skip_existing(context):
    logger.info("starting : Update reception tasks (skip existing)")
    if "standard" in utils.get_configs():
        utils.import_all_config(
            base_json_path="./profiles/config/standard",
            handle_existing_content=utils.ExistingContent.SKIP,
            match_filename="reception.json",
         )
    else:
        logger.info("nothing to upgrade")
    logger.info("upgrade done!")

          
def import_roaddecree_schedule_config(context):
    logger.info("starting : Import roaddecree tasks")
    if "liege" in utils.get_configs():
        utils.import_all_config(
            base_json_path="./profiles/config/liege",
            handle_existing_content=utils.ExistingContent.UPDATE,
            match_filename="decision-notification.json",
        )
    else:
        utils.import_all_config(
            base_json_path="./profiles/standard/default",
            handle_existing_content=utils.ExistingContent.UPDATE,
            match_filename="decision-notification.json",
        )
    logger.info("upgrade done!")



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import urban.schedule


class UrbanScheduleLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.

        self.loadZCML(package=urban.schedule)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "urban.schedule:default")


URBAN_SCHEDULE_FIXTURE = UrbanScheduleLayer()


URBAN_SCHEDULE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(URBAN_SCHEDULE_FIXTURE,),
    name="UrbanScheduleLayer:IntegrationTesting",
)


URBAN_SCHEDULE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(URBAN_SCHEDULE_FIXTURE,),
    name="UrbanScheduleLayer:FunctionalTesting",
)


URBAN_SCHEDULE_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        URBAN_SCHEDULE_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="UrbanScheduleLayer:AcceptanceTesting",
)



# -*- coding: utf-8 -*-

from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer
from urban.schedule.utils import import_all_config
from urban.schedule.utils import get_configs


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "urban.schedule:uninstall",
        ]


def post_install(context):
    """Post install script"""
    for config in get_configs():
        import_all_config(
            base_json_path="./profiles/config/{0}".format(config),
            base_context_path="portal_urban",
        )


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-

from collective.exportimport.import_content import ImportContent
from enum import Enum
from plone import api
from zope.component.hooks import getSite

import json
import os


class ExistingContent(Enum):
    SKIP = 0
    REPLACE = 1
    UPDATE = 2
    IGNORE = 3


def get_configs():
    """Return the configs defined in `URBAN_SCHEDULE_CONFIGS` env variable"""
    configs = os.environ.get("URBAN_SCHEDULE_CONFIGS", "default,standard")
    return configs.split(",")


def remove_uid(data):
    new_data = []

    for item in data:
        if "UID" in item:
            del item["UID"]
        if "UID" in item["parent"]:
            del item["parent"]["UID"]
        new_data.append(item)

    return new_data


def remove_none(data):
    return [{k: v for k, v in item.items() if v is not None} for item in data]


def fix_id(id):
    path = id.split("portal_urban")[1]
    root_site = getSite()
    return os.path.normpath(
        os.path.join(
            "/".join(root_site.getPhysicalPath()), "portal_urban", path.lstrip("/")
        )
    )


def fix_all_ids(data):
    new_data = []

    for item in data:
        item["@id"] = fix_id(item["@id"])
        item["parent"]["@id"] = fix_id(item["parent"]["@id"])
        new_data.append(item)

    return new_data


def import_json_config(
    json_path, context, handle_existing_content=ExistingContent.SKIP
):
    """
    This function is used to import a json file (exported with c.exportimport)

    :param json_path: Path to the json to be imported
    :type json_path: String
    :param context: Path to or object of the context where the json will be imported
    :type context: String or plone object
    :raises ValueError: Raise if the json doesn't exist
    """
    if not os.path.isfile(json_path):
        raise ValueError("{} does not exist".format(json_path))

    with open(json_path, "r") as f:
        data = json.load(f)

    portal = api.portal.get()

    if isinstance(context, str):
        context = portal.restrictedTraverse(context)

    request = getattr(context, "REQUEST", None)

    if request is None:
        request = portal.REQUEST

    import_content = ImportContent(context, request)

    import_content.import_to_current_folder = False
    import_content.handle_existing_content = handle_existing_content.value
    import_content.limit = None
    import_content.commit = None
    import_content.import_old_revisions = False

    data = remove_uid(data)
    data = remove_none(data)
    data = fix_all_ids(data)

    import_content.start()
    import_content.do_import(data)
    import_content.finish()


def import_all_config(
    base_json_path="./profiles/config",
    base_context_path="portal_urban",
    config_type="schedule",
    handle_existing_content=ExistingContent.SKIP,
    match_filename=None,
):
    """
    Function used to import all json inside a folder

    :param base_json_path: Root folder whre to find json, defaults to "./profiles/config"
    :type base_json_path: String, optional
    :param base_context_path: Path to or object of the context where the json will be
                              imported, defaults to "portal_urban"
    :type base_context_path: String or plone object, optional
    :param config_type: config folder where to import, defaults to "schedule"
    :type config_type: String, optional
    :param match_filename: a filename that will be use to restrict the imported configs,
                           defaults to "None"
    :type match_filename: String, optional
    """
    directory_path = os.path.dirname(os.path.realpath(__file__))

    licences_types = os.walk(
        os.path.normpath(os.path.join(directory_path, base_json_path))
    )

    root_site = getSite()

    for root, dirs, files in licences_types:
        if files == []:
            continue
        for file in files:
            if match_filename is not None and file != match_filename:
                continue
            json_path = os.path.join(root, file)
            licence_type = root.split("/")[-1]
            context_plone = os.path.normpath(
                os.path.join(
                    "/".join(root_site.getPhysicalPath()),
                    base_context_path,
                    licence_type,
                    config_type,
                )
            )
            import_json_config(
                json_path=json_path,
                context=context_plone,
                handle_existing_content=handle_existing_content,
            )
